/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: OSGiBean.java 3493 2008-06-13 22:08:22Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.osgi;

import java.util.ArrayList;
import java.util.List;

import javax.ejb.Stateless;
import javax.interceptor.AroundInvoke;
import javax.interceptor.InvocationContext;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.service.log.LogService;
import org.ow2.easybeans.osgi.annotation.BindingMode;
import org.ow2.easybeans.osgi.annotation.OSGiResource;

/**
 * EJB that can access the OSGi world.
 * @author Florent BENOIT
 */
@Stateless(mappedName="OSGiBean")
public class OSGiBean implements SimpleInterface {


    /**
     * Bundle context that will be injected by the container.
     */
    @OSGiResource
    private final BundleContext bundleContextField = null;

    /**
     * Bundle context that will be injected by the container by a setter method.
     */
    private BundleContext bundleContextMethod = null;

    /**
     * This dependency will be managed (injected/outjected) by OSGi in a method.
     */
    private ISimpleDependency dependencyMethod;

    /**
     * This OSGi dependency is directly injected/outjected in the field.
     * Notice that this interface is implemented by an EJB.
     */
    @OSGiResource
    private ISimpleDependency dependencyField;

    /**
     * This OSGi dependency is directly injected/outjected in the field.
     * Notice that this interface is implemented by a legacy OSGi service.
     */
    @OSGiResource
    private LogService logger;

    /**
     * Sets the bundleContext provided by the container.
     * @param bundleContext the given bundle context
     */
    @OSGiResource
    public void setBundleContext(final BundleContext bundleContext) {
        System.out.println("Receiving bundlecontext from the container:" + bundleContext);
        this.bundleContextMethod = bundleContext;
    }

    /**
     * @return the list of all OSGi Bundles symbolic names found on the gateway.
     */
    public List<String> listOSGiSymbolicNameBundles() {
        List<String> symbolicNames = new ArrayList<String>();
        Bundle[] bundles = bundleContextMethod.getBundles();
        if (bundles != null) {
            for (Bundle bundle : bundles) {
                symbolicNames.add(bundle.getSymbolicName());
            }
        }
        return symbolicNames;
    }

    /**
     * Try to log on the OSGi Log service.
     * @return result of the commands.
     */
    public String tryLogService() {
        StringBuilder sb = new StringBuilder();
        if (logger == null) {
            sb.append("No log service reference found");
        } else {
            sb.append("Found log service reference '" + logger + "'.");
            logger.log(LogService.LOG_INFO, "This is a test of a log message from an EJB 3.0 on an OSGi service");
            sb.append("Log on the logservice has been done");
        }
        // Display log on the server side
        System.out.println(sb.toString());
        return sb.toString();
    }

    /**
     * EJB 3.0 interceptor.
     * @param invocationContext the invocation context of the EJB.
     * @return the result of the invocation.
     * @throws Exception if the interceptor is failing
     */
    @AroundInvoke
    protected Object intercepted(final InvocationContext invocationContext) throws Exception {
        System.out.println("EJB 3.0 interceptor on the method '" + invocationContext.getMethod() + "'.");
        return invocationContext.proceed();
    }

    /**
     * Injection method for the {@link ISimpleDependency} interface.
     * @param dep the bean implementing the interface
     */
    @OSGiResource(mode=BindingMode.BIND)
    public void bindDependency(final ISimpleDependency dep) {
        System.out.println(ISimpleDependency.class.getName() + " has been injected.");
        this.dependencyMethod = dep;
    }

    /**
     * Outjection method for the {@link ISimpleDependency} interface.
     * @param dep the bean implementing the interface
     */
    @OSGiResource(mode=BindingMode.UNBIND)
    public void unbindLogService(final ISimpleDependency dep) {
        System.out.println(ISimpleDependency.class.getName() + " has been removed.");
        this.dependencyMethod = null;
    }

    /**
     * A test method checking if the dependency has been injected in the field.
     * @return <code>true</code> if injected
     */
    public boolean isDependencyFieldInjected() {
        return dependencyField != null;
    }

    /**
     * A test method checking if the dependency has been injected through the methods.
     * @return <code>true</code> if injected
     */
    public boolean isDependencyMethodInjected() {
        return dependencyMethod != null;
    }
}
